'use strict';

module.exports = {
  rules: {

    // enforce consistent spacing inside array brackets
    'array-bracket-spacing': ['warn', 'never', {
      arraysInArrays: false,
      objectsInArrays: false,
      singleValue: false,
    }],

    // enforce consistent spacing inside single-line blocks
    'block-spacing': ['warn', 'always'],

    // enforce consistent brace style for blocks
    'brace-style': ['warn', '1tbs', {
      allowSingleLine: true,
    }],

    // enforce camelcase naming convention
    camelcase: ['warn', {
      properties: 'never',
    }],

    // require or disallow trailing commas
    'comma-dangle': ['warn', 'always-multiline'],

    // enforce consistent spacing before and after commas
    'comma-spacing': ['warn', {
      after: true,
      before: false,
    }],

    // enforce consistent comma style
    'comma-style': ['warn', 'last', {
      exceptions: {
        ArrayExpression: false,
        ObjectExpression: false,
        VariableDeclaration: false,
      },
    }],

    // enforce consistent spacing inside computed property brackets
    'computed-property-spacing': ['warn', 'never'],

    // enforce consistent naming when capturing the current execution context
    'consistent-this': 'off',

    // enforce at least one newline at the end of files
    'eol-last': 'warn',

    // require or disallow spacing between function identifiers and their invocations
    'func-call-spacing': ['warn', 'never'],

    // require or disallow named function expressions
    'func-names': 'off',

    // enforce the consistent use of either function declarations or expressions
    'func-style': 'off',

    // disallow specified identifiers
    'id-blacklist': 'off',

    // enforce minimum and maximum identifier lengths
    'id-length': ['warn', {
      min: 2,
      properties: 'always',
    }],

    // require identifiers to match a specified regular expression
    'id-match': 'off',

    // enforce consistent indentation
    indent: ['warn', 2, {
      outerIIFEBody: 1,
      SwitchCase: 1,
      VariableDeclarator: 1,
    }],

    // enforce the consistent use of either double or single quotes in JSX attributes
    'jsx-quotes': 'off',

    // enforce consistent spacing between keys and values in object literal properties
    'key-spacing': ['warn', {
      afterColon: true,
      beforeColon: false,
      mode: 'strict',
    }],

    // enforce consistent spacing before and after keywords
    'keyword-spacing': ['warn', {
      after: true,
      before: true,
      overrides: {},
    }],

    // enforce position of line comments
    'line-comment-position': 'off',

    // enforce consistent linebreak style
    'linebreak-style': ['error', 'unix'],

    // require empty lines around comments
    'lines-around-comment': ['warn', {
      beforeBlockComment: true,
      beforeLineComment: true,
    }],

    // require or disallow newlines around directives
    'lines-around-directive': 'off',

    // enforce a maximum depth that blocks can be nested
    'max-depth': ['warn', 4],

    // enforce a maximum line length
    'max-len': ['warn', {
      code: 100,
      ignoreComments: false,
      ignoreStrings: false,
      ignoreUrls: true,
      tabWidth: 2,
    }],

    // enforce a maximum number of lines per file
    'max-lines': 'off',

    // enforce a maximum depth that callbacks can be nested
    'max-nested-callbacks': 'off',

    // enforce a maximum number of parameters in function definitions
    'max-params': 'off',

    // enforce a maximum number of statements allowed in function blocks
    'max-statements': 'off',

    // enforce a maximum number of statements allowed per line
    'max-statements-per-line': ['warn', {
      max: 1,
    }],

    // enforce newlines between operands of ternary expressions
    'multiline-ternary': ['warn', 'never'],

    // require constructor function names to begin with a capital letter
    'new-cap': ['warn', {
      capIsNew: false,
      newIsCap: true,
      properties: false,
    }],

    // require parentheses when invoking a constructor with no arguments
    'new-parens': 'warn',

    // require or disallow an empty line after var declarations
    'newline-after-var': 'off',

    // require an empty line before return statements
    'newline-before-return': 'off',

    // require a newline after each call in a method chain
    'newline-per-chained-call': ['warn', {
      ignoreChainWithDepth: 4,
    }],

    // disallow Array constructors
    'no-array-constructor': 'warn',

    // disallow bitwise operators
    'no-bitwise': 'error',

    // disallow continue statements
    'no-continue': 'warn',

    // disallow inline comments after code
    'no-inline-comments': 'off',

    // disallow if statements as the only statement in else blocks
    'no-lonely-if': 'warn',

    // disallow mixed binary operators
    'no-mixed-operators': ['error', {
      groups: [
        ['&&', '||'],
        ['in', 'instanceof'],
        ['+', '-', '*', '/', '%', '**'],
        ['&', '|', '^', '~', '<<', '>>', '>>>'],
        ['==', '!=', '===', '!==', '>', '>=', '<', '<='],
      ],
    }],

    // disallow mixed spaces and tabs for indentation
    'no-mixed-spaces-and-tabs': 'error',

    // disallow multiple empty lines
    'no-multiple-empty-lines': ['warn', {
      max: 1,
      maxBOF: 1,
      maxEOF: 1,
    }],

    // disallow negated conditions
    'no-negated-condition': 'off',

    // disallow nested ternary expressions
    'no-nested-ternary': 'warn',

    // disallow Object constructors
    'no-new-object': 'warn',

    // disallow the unary operators ++ and --
    'no-plusplus': ['error', {
      allowForLoopAfterthoughts: false,
    }],

    // disallow specified syntax
    'no-restricted-syntax': ['error', 'ForInStatement', 'LabeledStatement', 'WithStatement'],

    // disallow tabs in file
    'no-tabs': 'error',

    // disallow ternary operators
    'no-ternary': 'off',

    // disallow trailing whitespace at the end of lines
    'no-trailing-spaces': ['warn', {
      skipBlankLines: false,
    }],

    // disallow dangling underscores in identifiers
    'no-underscore-dangle': ['warn', {
      allow: ['_href', '_id', '_link', '_type'],
      allowAfterSuper: false,
      allowAfterThis: false,
    }],

    // disallow ternary operators when simpler alternatives exist
    'no-unneeded-ternary': ['warn', {
      defaultAssignment: false,
    }],

    // disallow whitespace before properties
    'no-whitespace-before-property': 'warn',

    // enforce consistent line breaks inside braces
    'object-curly-newline': 'off',

    // enforce consistent spacing inside braces
    'object-curly-spacing': ['warn', 'always', {
      arraysInObjects: true,
      objectsInObjects: true,
    }],

    // enforce placing object properties on separate lines
    'object-property-newline': ['warn', {
      allowMultiplePropertiesPerLine: true,
    }],

    // enforce variables to be declared either together or separately in functions
    'one-var': ['warn', 'never'],

    // require or disallow newlines around var declarations
    'one-var-declaration-per-line': ['warn', 'always'],

    // require or disallow assignment operator shorthand where possible
    'operator-assignment': ['warn', 'always'],

    // enforce consistent linebreak style for operators
    'operator-linebreak': 'off',

    // require or disallow padding within blocks
    'padded-blocks': 'off',

    // require quotes around object literal property names
    'quote-props': ['warn', 'as-needed', {
      keywords: false,
      numbers: false,
      unnecessary: true,
    }],

    // enforce the consistent use of either backticks, double, or single quotes
    quotes: ['warn', 'single', {
      avoidEscape: false,
    }],

    // require JSDoc comments
    'require-jsdoc': 'off',

    // require or disallow semicolons instead of ASI
    semi: ['warn', 'always'],

    // enforce consistent spacing before and after semicolons
    'semi-spacing': ['warn', {
      after: true,
      before: false,
    }],

    // requires object keys to be sorted
    'sort-keys': ['warn', 'asc', {
      caseSensitive: false,
      natural: true,
    }],

    // require variables within the same declaration block to be sorted
    'sort-vars': 'off',

    // enforce consistent spacing before blocks
    'space-before-blocks': ['warn', 'always'],

    // enforce consistent spacing before function definition opening parenthesis
    'space-before-function-paren': ['warn', {
      anonymous: 'always',
      named: 'never',
    }],

    // enforce consistent spacing inside parentheses
    'space-in-parens': ['warn', 'never', {
      exceptions: [],
    }],

    // require spacing around operators
    'space-infix-ops': ['warn', {
      int32Hint: false,
    }],

    // enforce consistent spacing before or after unary operators
    'space-unary-ops': ['warn', {
      nonwords: false,
      overrides: {},
      words: true,
    }],

    // enforce consistent spacing after the // or /* in a comment
    'spaced-comment': ['warn', 'always', {
      exceptions: ['+', '-'],
      markers: ['!', '='],
    }],

    // require or disallow Unicode byte order mark (BOM)
    'unicode-bom': ['error', 'never'],

    // require parenthesis around regex literals
    'wrap-regex': 'off',
  },
};
