'use strict';

module.exports = {
  rules: {

    // require braces around arrow function bodies
    'arrow-body-style': ['warn', 'as-needed', {
      requireReturnForObjectLiteral: false,
    }],

    // require parentheses around arrow function arguments
    'arrow-parens': ['warn', 'as-needed', {
      requireForBlockBody: true,
    }],

    // enforce consistent spacing before and after the arrow in arrow functions
    'arrow-spacing': ['warn', {
      after: true,
      before: true,
    }],

    // require super() calls in constructors
    'constructor-super': 'error',

    // enforce consistent spacing around * operators in generator functions
    'generator-star-spacing': ['warn', {
      after: true,
      before: false,
    }],

    // disallow reassigning class members
    'no-class-assign': 'error',

    // disallow arrow functions where they could be confused with comparisons
    'no-confusing-arrow': ['warn', {
      allowParens: true,
    }],

    // disallow reassigning const variables
    'no-const-assign': 'error',

    // disallow duplicate class members
    'no-dupe-class-members': 'error',

    // disallow duplicate module imports
    'no-duplicate-imports': ['warn', {
      includeExports: false,
    }],

    // disallow new operators with the Symbol object
    'no-new-symbol': 'error',

    // disallow specified modules when loaded by import
    'no-restricted-imports': 'off',

    // disallow this/super before calling super() in constructors
    'no-this-before-super': 'error',

    // disallow unnecessary computed property keys in object literals
    'no-useless-computed-key': 'warn',

    // disallow unnecessary constructors
    'no-useless-constructor': 'warn',

    // disallow renaming import, export, and destructured assignments to the same name
    'no-useless-rename': ['warn', {
      ignoreDestructuring: false,
      ignoreExport: false,
      ignoreImport: false,
    }],

    // require let or const instead of var
    'no-var': 'error',

    // require or disallow method and property shorthand syntax for object literals
    'object-shorthand': ['warn', 'always', {
      avoidQuotes: true,
      ignoreConstructors: false,
    }],

    // require arrow functions as callbacks
    'prefer-arrow-callback': ['warn', {
      allowNamedFunctions: false,
      allowUnboundThis: true,
    }],

    // require const declarations for variables that are never reassigned after declared
    'prefer-const': ['error', {
      destructuring: 'any',
      ignoreReadBeforeAssign: true,
    }],

    // disallow parseInt() in favor of binary, octal, and hexadecimal literals
    'prefer-numeric-literals': 'off',

    // require Reflect methods where applicable
    'prefer-reflect': 'off',

    // require rest parameters instead of arguments
    'prefer-rest-params': 'error',

    // require spread operators instead of .apply()
    'prefer-spread': 'warn',

    // require template literals instead of string concatenation
    'prefer-template': 'warn',

    // require generator functions to contain yield
    'require-yield': 'error',

    // enforce spacing between rest and spread operators and their expressions
    'rest-spread-spacing': ['warn', 'never'],

    // enforce sorted import declarations within modules
    'sort-imports': ['warn', {
      ignoreCase: true,
      ignoreMemberSort: false,
      memberSyntaxSortOrder: ['none', 'all', 'multiple', 'single'],
    }],

    // require symbol descriptions
    'symbol-description': 'warn',

    // require or disallow spacing around embedded expressions of template strings
    'template-curly-spacing': ['warn', 'never'],

    // require or disallow spacing around the * in yield* expressions
    'yield-star-spacing': ['warn', {
      after: true,
      before: false,
    }],
  },
};
