import fs from 'fs';
import path from 'path';

export function getConfigRules() {
  const rulesTree = [
    {
      name: 'es5',
      rules: [
        'bestPractices',
        'errors',
        'stylistic',
        'variables',
      ],
    },
    {
      name: 'es6',
      rules: [
        'bestPractices',
        'errors',
        'es6',
        'stylistic',
        'variables',
      ],
    },
    {
      name: 'node',
      rules: [
        'errors',
        'node',
      ],
    },
    {
      name: 'react',
      rules: [
        'stylistic',
      ],
    },
  ];

  return rulesTree.map((config) => {
    const mappedConfig = { name: config.name };

    mappedConfig.rules = config.rules.map(rule => ({
      module: require(path.join('..', '..', 'rules', config.name, rule)),
      name: rule,
    }));

    return mappedConfig;
  });
}

export function getESLintRules() {
  const rulesDirectory = path.join('node_modules', 'eslint', 'lib', 'rules');

  const ruleFiles = fs
    .readdirSync(rulesDirectory)
    .filter(file => file.endsWith('.js'))
    .map(file => file.replace(/\.js$/, ''));

  return ruleFiles.map(rule => ({
    module: require(path.resolve(rulesDirectory, rule)),
    name: rule,
  }));
}
