import { getConfigRules, getESLintRules } from './modules/getRules';
import { expect } from 'chai';

describe('Rules', () => {
  const configRules = getConfigRules();
  const eslintRules = getESLintRules();

  for (const config of configRules) {
    describe(config.name, () => {
      for (const rules of config.rules) {

        // Rule file assertions
        describe(rules.name, () => {
          it('is an object', () => {
            expect(rules.module).to.be.an('object');
          });

          it('has a rules property', () => {
            expect(rules.module).to.have.property('rules');
          });

          specify('rules property is an object', () => {
            expect(rules.module.rules).to.be.an('object');
          });

          specify('rules property is not empty', () => {
            expect(rules.module.rules).to.not.be.empty;
          });

          for (const rule of Object.keys(rules.module.rules)) {

            // Individual rule assertions
            describe(rule, () => {
              const ruleValue = rules.module.rules[rule];
              const ruleSettings = ['off', 'warn', 'error'];
              const ruleDefinition = eslintRules.filter(eslintRule => eslintRule.name === rule);

              it('is a valid base ESLint rule', () => {
                expect(ruleDefinition).to.be.an('array');
                expect(ruleDefinition).to.not.be.empty;
                expect(ruleDefinition[0]).to.be.an('object');
                expect(ruleDefinition[0]).to.have.property('module');
                expect(ruleDefinition[0].module).to.be.an('object');
              });

              it('is not a deprecated rule', () => {
                const ruleModule = ruleDefinition[0].module;

                expect(ruleModule).to.have.property('meta');
                expect(ruleModule.meta).to.be.an('object');

                const deprecated = Boolean(ruleModule.meta.deprecated);
                expect(deprecated).to.be.false;
              });

              it('is an array or a string', () => {
                if (Array.isArray(ruleValue)) {
                  expect(ruleValue).to.be.an('array');
                } else {
                  expect(ruleValue).to.be.a('string');
                }
              });

              if (Array.isArray(ruleValue)) {
                it('begins with a rule setting string', () => {
                  expect(ruleSettings).to.include(ruleValue[0]);
                });

              } else {
                it('is a rule setting string', () => {
                  expect(ruleSettings).to.include(ruleValue);
                });
              }
            });
          }
        });
      }
    });
  }
});
